<?php

namespace GopNext\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;

class SpreadsheetImportController {
    
    protected $crudService;
    protected $service;
    
    public function __construct($connection, $environment, $util, $crudService, $service) {
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->util         = $util;
        $this->crudService	= $crudService;
        $this->service      = $service;
    }
    
    public function importBudget(Request\Row $request, Response $response) {
        $rows = $request->getRow();
        
        try {
            $this->service->beginTransaction();
            
            foreach ($rows as &$row) {
                if (strlen($row['CDFILIAL']) === 4) {
                    $params = [
                        "CDFILIAL"  => $row['CDFILIAL'],
                        "NRANO"     => $row['NRANO'],
                        "CDITMORCA" => $row['CDORCAMENTO'],
                        "CDDESPDIV" => $row['CODIGO'] ?? null,
                        "CDARVPROD" => $row['CDARVPROD'] ?? null,
                        "VRORCAJAN" => $row['JANEIRO'],
                        "VRORCAFEV" => $row['FEVEREIRO'],
                        "VRORCAMAR" => $row['MARCO'],
                        "VRORCAABR" => $row['ABRIL'],
                        "VRORCAMAI" => $row['MAIO'],
                        "VRORCAJUN" => $row['JUNHO'],
                        "VRORCAJUL" => $row['JULHO'],
                        "VRORCAAGO" => $row['AGOSTO'],
                        "VRORCASET" => $row['SETEMBRO'],
                        "VRORCAOUT" => $row['OUTUBRO'],
                        "VRORCANOV" => $row['NOVEMBRO'],
                        "VRORCADEZ" => $row['DEZEMBRO'],
                        "IDORCAIMP" => 'N'
                    ];
                    
                    $this->service->recordContent($params);
                }
            };
            
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }

/*
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        $dataDiet = [];
        
        $data = $this->crudService->getFilteredDataSet('gop10000_serviceDetails', array_merge($conditions, $filter))->getRows();
        
        if (isset($data[0])) {
            $conditions = [
                array(
                    "columnName"   => 'CDFILIAL',
                    "value" => $data[0]['CDFILIAL'],
                    "operator" => "="
                ),
                array(
                    "columnName"   => 'CDSERVFILI',
                    "value" => $data[0]['CDSERVFILI'],
                    "operator" => "="
                )
            ];
            
            $dataDiet = $this->crudService->getFilteredDataSet('gop10000_serviceDetailsDiet', array_merge($conditions, $filter))->getRows();
        }
		$response->addDataSet(new DataSet("gop10000_serviceDetails", $data));
		*/
		$response->addDataSet(new DataSet("importBudget", [$rows]));
    }
    
    function removeProperty(array &$array, string $property): array {
        foreach ($array as &$object) {
            if (array_key_exists($property, $object)) {
                unset($object[$property]);
            }
        }
        return $array;
    }
    
    function includesHeader(array &$array): array {
        $newObj = [
            'CDFILIAL' => 'Unidade',
            'NRANO' => 'Ano',
            'CDORCAMENTO' => 'Código Orçamento',
            'NMORCAMENTO' => 'Nome Orçamento',
            'CODIGO' => 'Código',
            'DESCRICAO' => 'Descrição',
            'CDARVPROD' => 'Código Detalhamento',
            'NMPRODUTO' => 'Nome Detalhamento',
            'JANEIRO' => 'Janeiro',
            'FEVEREIRO' => 'Fevereiro',
            'MARCO' => 'Março',
            'ABRIL' => 'Abril',
            'MAIO' => 'Maio',
            'JUNHO' => 'Junho',
            'JULHO' => 'Julho',
            'AGOSTO' => 'Agosto',
            'SETEMBRO' => 'Setembro',
            'OUTUBRO' => 'Outubro',
            'NOVEMBRO' => 'Novembro',
            'DEZEMBRO' => 'Dezembro'
        ];
        array_unshift($array, $newObj);
        
        return $array;
    }
    
    public function exportBudget(Request\DataSet $request, Response $response) {
        $filter = $request->getParameters();
        $params = ['CDOPERADOR'];
        $oper   = $this->util->prepareConditions($params);
        $nrLista = $this->util->insertLista($filter['CDFILIAL']);
        $data = [];
        
        $conditions = [
            array(
                "columnName" => 'NRLISTA',
                "value"      => $nrLista,
                "operator"   => "="
            ),
            array(
                "columnName" => 'NRANO',
                "value"      => $filter['NRANO'],
                "operator"   => "="
            ),
            array(
                "columnName" => 'STARTVALUE',
                "value"      => $filter['STARTVALUE'] ? 0 : null,
                "operator"   => "="
            )
        ];
        
        try {
            if ($filter['MODELO'] === 'S') {
                $data = $this->crudService->getFilteredDataSet('gop10000_exportModelBudgetDetail', array_merge($conditions, $oper))->getRows();
                
    		    $data = $this->includesHeader($data);
    		    $data = $this->includesHeader($data);
            } else {
                $data = $this->crudService->getFilteredDataSet('gop10000_exportModelBudget', array_merge($conditions, $oper))->getRows();
                
    		    $data = $this->includesHeader($data);
    		    $data = $this->includesHeader($data);
                
                $data = $this->removeProperty($data, 'CDARVPROD');
                $data = $this->removeProperty($data, 'NMPRODUTO');
                $data = $this->removeProperty($data, 'CODIGO');
                $data = $this->removeProperty($data, 'DESCRICAO');
            }
            
    		$this->util->deleteLista($nrLista);
    		
    		$data = $this->removeProperty($data, '__is_new');
    		$data = $this->removeProperty($data, 'NRORG');
    		
    		$response->addDataSet(new DataSet("exportBudget", $data));
        } catch(\Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
}
