import {
  Modal,
  ModalService,
  Select,
} from '@zeedhi/common';
import {
  Metadata,
  IEventParam,
} from '@zeedhi/core';
import {
  TekGrid,
} from '@zeedhi/teknisa-components-common';

export class DetailsItemOperationalModalController {
  private detailsItemOperationalModal!: Modal;
  
  public rowsEditableDetail = [];
  
  public async createDetailsItemOperationalModal() {
    this.detailsItemOperationalModal = await ModalService.createFromJson('/modals/detailsItemOperationalModal', true);
  }
  
  public destroyDetailsItemOperationalModal() {
    this.detailsItemOperationalModal?.destroy();
  }
  
  public async openDetailsItemOperationalModal({ row }) {
    const filter = {
      CDFILIAL: row.CDFILIAL,
      NRPERCENTUAL: row.NRSEQUENCIAL,
      ORIGEM: row.DSORIGEM,
      CDDESPPREV: row.CDCODIGO,
    };
    await this.detailsItemOperationalModal.show();
    const gridDetailModal = Metadata.getInstance<TekGrid>('gridDetailsItemOperational');
    gridDetailModal.datasource.setFilter(filter);
  }
  
  public closeDetailsItemOperationalModal() {
    this.detailsItemOperationalModal.hide();
  }
  
  /* public setRouteDetailModal() {
    const gridUnits = Metadata.getInstance<TekGrid>('gridUnits');
    const gridItems = Metadata.getInstance<TekGrid>('gridOperationalItems');
    // const originRow = row.DSORIGEM;
    const cdFilial = gridUnits.datasource.currentRow.CDFILIAL;
    const nrSeqItem = gridItems.datasource.currentRow.NRSEQITEMORCA;
    console.warn(nrSeqItem);
    const cdDespPrev = '';
    const nrPercentual = '';
    const routeDescription = [
      // ORIGEM DESPESAS
      {
        ORIGIN: 'DESPESA',
        route: '/crudN/detailExpense',
        params: {
          CDDESPPREV: cdDespPrev,
          NRPERCENTUAL: nrPercentual,
          ORIGEM: 'DESPESA',
          CDFILIAL: cdFilial,
        },
      },
      // ORIGEM MAO DE OBRA
      {
        ORIGIN: 'ORCAMENTOMO', route: '/crudN/detailLabor', params: { CDFILIAL: cdFilial },
      },
    ];
    const result = routeDescription.filter((item) => item.ORIGIN === 'DESPESAS');
    const route = result[0].route;
    const params = result[0].params;
    return {
      route,
      uniqueKey: 'CDCODIGO',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: params,
    };
  } */
  
  public setRouteDescriptionDetail({ row, component }) {
    const type = component ? component.value : row.TIPO;
    const routeDescription = [
      // PRODUTO
      {
        TYPE: 'PRODUTO', route: '/crudN/descriptionProductDetail', params: {},
      },
      // CONTA CONTABIL
      {
        TYPE: 'CONTABIL', route: '/crudN/descriptionAccountDetail', params: {},
      },
      // CONTA A PAGAR
      {
        TYPE: 'TCPAGAR', route: '/crudN/descriptionBillsToPayDetail', params: {},
      },
    ];
    const result = routeDescription.filter((item) => item.TYPE === type);
    const route = result.length > 0 ? result[0].route : '';
    const params = result.length > 0 ? result[0].params : {};
    /* if (component) {
      const gridDetail = Metadata.getInstance<TekGrid>('detailsOperationalItems');
      const fieldDescription = gridDetail.getComponent(row.NRSEQUENCIAL, 'DSDESCRICAO');
      (fieldDescription as any).datasource.route = route;
      (fieldDescription as any).datasource.filter = params;
    } */
    return {
      route,
      uniqueKey: 'CODIGOARV',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: params,
    };
  }
  
  public addNewRow({ component }: IEventParam<TekGrid>) {
    const newId = (component.datasource.total + 1).toString();
    this.rowsEditableDetail.push(newId);
    component.addNewRow({ NRSEQ: newId }, 'start');
    const nameInput = Metadata.getInstance<Select>(`TIPO_editable_${newId}`);
    nameInput.setFocus();
  }
  
  public editableType({ row }) {
    const newRow = this.rowsEditableDetail.indexOf(row.NRSEQ) !== -1;
    return newRow;
  }
}
